@@ -25,6 +25,7 @@
 #include "defargs.h"
 
 static std::mutex g_cacheMutex;
+static std::recursive_mutex g_cacheTypedefMutex;
 
 //--------------------------------------------------------------------------------------
 
@@ -143,12 +144,13 @@ struct SymbolResolver::Private
 
     const Definition *followPath(const Definition *start,const QCString &path);
 
-    const Definition *endOfPathIsUsedClass(LinkedRefMap<const ClassDef> cl,const QCString &localName);
+    const Definition *endOfPathIsUsedClass(const LinkedRefMap<const ClassDef> &cl,const QCString &localName);
 
     bool accessibleViaUsingNamespace(StringUnorderedSet &visited,
                                      const LinkedRefMap<const NamespaceDef> &nl,
                                      const Definition *item,
-                                     const QCString &explicitScopePart="");
+                                     const QCString &explicitScopePart="",
+                                     int level=0);
     bool accessibleViaUsingClass(const LinkedRefMap<const ClassDef> &cl,
                                  const Definition *item,
                                  const QCString &explicitScopePart=""
@@ -257,10 +259,9 @@ const ClassDef *SymbolResolver::Private::getResolvedClassRec(
   }
   *pk='\0';
 
-  LookupInfo *pval = 0;
   {
     std::lock_guard<std::mutex> lock(g_cacheMutex);
-    pval=Doxygen::lookupCache->find(key.str());
+    LookupInfo *pval = Doxygen::lookupCache->find(key.str());
     //printf("Searching for %s result=%p\n",qPrint(key),pval);
     if (pval)
     {
@@ -279,7 +280,7 @@ const ClassDef *SymbolResolver::Private::getResolvedClassRec(
     else // not found yet; we already add a 0 to avoid the possibility of
       // endless recursion.
     {
-      pval = Doxygen::lookupCache->insert(key.str(),LookupInfo());
+      Doxygen::lookupCache->insert(key.str(),LookupInfo());
     }
   }
 
@@ -312,13 +313,11 @@ const ClassDef *SymbolResolver::Private::getResolvedClassRec(
   //printf("getResolvedClassRec: bestMatch=%p pval->resolvedType=%s\n",
   //    bestMatch,qPrint(bestResolvedType));
 
-  if (pval)
   {
+    // we need to insert the item in the cache again, as it could be removed in the meantime
     std::lock_guard<std::mutex> lock(g_cacheMutex);
-    pval->classDef     = bestMatch;
-    pval->typeDef      = bestTypedef;
-    pval->templSpec    = bestTemplSpec;
-    pval->resolvedType = bestResolvedType;
+    Doxygen::lookupCache->insert(key.str(),
+                          LookupInfo(bestMatch,bestTypedef,bestTemplSpec,bestResolvedType));
   }
   //fprintf(stderr,"%d ] bestMatch=%s distance=%d\n",--level,
   //    bestMatch?qPrint(bestMatch->name()):"<none>",minDistance);
@@ -494,6 +493,7 @@ const ClassDef *SymbolResolver::Private::newResolveTypedef(
                   QCString *pResolvedType,                             // out
                   const std::unique_ptr<ArgumentList> &actTemplParams) // in
 {
+  std::lock_guard<std::recursive_mutex> lock(g_cacheTypedefMutex);
   //printf("newResolveTypedef(md=%p,cachedVal=%p)\n",md,md->getCachedTypedefVal());
   bool isCached = md->isTypedefValCached(); // value already cached
   if (isCached)
@@ -818,7 +818,7 @@ const Definition *SymbolResolver::Private::followPath(const Definition *start,co
   return current; // path could be followed
 }
 
-const Definition *SymbolResolver::Private::endOfPathIsUsedClass(LinkedRefMap<const ClassDef> cl,const QCString &localName)
+const Definition *SymbolResolver::Private::endOfPathIsUsedClass(const LinkedRefMap<const ClassDef> &cl,const QCString &localName)
 {
   for (const auto &cd : cl)
   {
@@ -833,35 +833,34 @@ const Definition *SymbolResolver::Private::endOfPathIsUsedClass(LinkedRefMap<con
 bool SymbolResolver::Private::accessibleViaUsingNamespace(StringUnorderedSet &visited,
                                  const LinkedRefMap<const NamespaceDef> &nl,
                                  const Definition *item,
-                                 const QCString &explicitScopePart)
+                                 const QCString &explicitScopePart,
+                                 int level)
 {
+  //size_t count=0;
   for (const auto &und : nl) // check used namespaces for the class
   {
-    //printf("[Trying via used namespace %s: count=%d/%d\n",qPrint(und->name()),
-    //    count,nl->count());
+    //printf("%d [Trying via used namespace %s: count=%zu/%zu\n",level,qPrint(und->name()),
+    //    count++,nl.size());
     const Definition *sc = explicitScopePart.isEmpty() ? und : followPath(und,explicitScopePart);
     if (sc && item->getOuterScope()==sc)
     {
-      //printf("] found it\n");
+      //printf("%d ] found it\n",level);
       return true;
     }
     if (item->getLanguage()==SrcLangExt_Cpp)
     {
-      QCString key=und->name();
-      if (!und->getUsedNamespaces().empty() && visited.find(key.str())==visited.end())
+      QCString key=und->qualifiedName();
+      if (!und->getUsedNamespaces().empty() && visited.insert(key.str()).second)
       {
-        visited.insert(key.str());
-
-        if (accessibleViaUsingNamespace(visited,und->getUsedNamespaces(),item,explicitScopePart))
+        if (accessibleViaUsingNamespace(visited,und->getUsedNamespaces(),item,explicitScopePart,level+1))
         {
-          //printf("] found it via recursion\n");
+          //printf("%d ] found it via recursion\n",level);
           return true;
         }
 
-        visited.erase(key.str());
       }
     }
-    //printf("] Try via used namespace done\n");
+    //printf("%d ] Try via used namespace done\n",level);
   }
   return false;
 }
@@ -950,7 +949,7 @@ int SymbolResolver::Private::isAccessibleFrom(AccessStack &accessStack,
         goto done;
       }
       StringUnorderedSet visited;
-      if (accessibleViaUsingNamespace(visited,nscope->getUsedNamespaces(),item))
+      if (accessibleViaUsingNamespace(visited,nscope->getUsedNamespaces(),item,0))
       {
         //printf("> found via used namespace\n");
         goto done;
@@ -1039,6 +1038,7 @@ const ClassDef *SymbolResolver::resolveClass(const Definition *scope,
       (scope->definitionType()!=Definition::TypeClass &&
        scope->definitionType()!=Definition::TypeNamespace
       ) ||
+      (name.stripWhiteSpace().startsWith("::")) ||
       (scope->getLanguage()==SrcLangExt_Java && QCString(name).find("::")!=-1)
      )
   {
@@ -1046,7 +1046,7 @@ const ClassDef *SymbolResolver::resolveClass(const Definition *scope,
   }
   //fprintf(stderr,"------------ resolveClass(scope=%s,name=%s,mayUnlinkable=%d)\n",
   //    scope?qPrint(scope->name()):"<global>",
-  //    name,
+  //    qPrint(name),
   //    mayBeUnlinkable
   //   );
   const ClassDef *result;
@@ -1068,12 +1068,12 @@ const ClassDef *SymbolResolver::resolveClass(const Definition *scope,
   {
     if (!mayBeHidden || !result->isHidden())
     {
-      //printf("result was %s\n",result?qPrint(result->name()):"<none>");
+      //fprintf(stderr,"result was %s\n",result?qPrint(result->name()):"<none>");
       result=0; // don't link to artificial/hidden classes unless explicitly allowed
     }
   }
   //fprintf(stderr,"ResolvedClass(%s,%s)=%s\n",scope?qPrint(scope->name()):"<global>",
-  //                                  name,result?qPrint(result->name()):"<none>");
+  //                                  qPrint(name),result?qPrint(result->name()):"<none>");
   return result;
 }
 
