@@ -17,16 +17,22 @@
 #define TEMPLATE_H
 
 #include <vector>
+#include <memory>
+#include <functional>
+#include <variant>
 
 #include "qcstring.h"
 #include "containers.h"
-#include "variant.h"
 
 class TemplateListIntf;
 class TemplateStructIntf;
 class TemplateEngine;
 class TextStream;
 
+using TemplateListIntfPtr       = std::shared_ptr<TemplateListIntf>;
+using TemplateStructIntfPtr     = std::shared_ptr<TemplateStructIntf>;
+using TemplateStructIntfWeakPtr = std::weak_ptr<TemplateStructIntf>;
+
 /** @defgroup template_api Template API
  *
  *  This is the API for a
@@ -92,78 +98,75 @@ class TextStream;
 class TemplateVariant
 {
   public:
-    /** @brief Helper class to create a delegate that can store a function/method call. */
-    class FunctionDelegate
+    /** Type representing a function call in a template */
+    using FunctionDelegate = std::function<TemplateVariant(const std::vector<TemplateVariant>&)>;
+
+    /** Symbolic names for the possible types that this variant can hold. */
+    using VariantT = std::variant<std::monostate,            // index==0, Invalid/default type
+                                  bool,                      // index==1: Type::Bool
+                                  int,                       // index==2: Type::Int
+                                  QCString,                  // index==3: Type::String
+                                  TemplateStructIntfPtr,     // index==4: Type::Struct
+                                  TemplateListIntfPtr,       // index==5: Type::List
+                                  FunctionDelegate,          // index==6: Type::Function
+                                  TemplateStructIntfWeakPtr  // index==7: Type::WeakStruct
+                                 >;
+
+    enum class Type : uint8_t
     {
-      public:
-        /** Callback type to use when creating a delegate from a function. */
-        typedef TemplateVariant (*StubType)(const void *obj, const std::vector<TemplateVariant> &args);
-
-        FunctionDelegate() : m_objectPtr(0) , m_stubPtr(0) {}
-
-        /** Creates a delegate given an object. The method to call is passed as a template parameter */
-        template <class T, TemplateVariant (T::*TMethod)(const std::vector<TemplateVariant> &) const>
-        static FunctionDelegate fromMethod(const T* objectPtr)
-        {
-          FunctionDelegate d;
-          d.m_objectPtr = objectPtr;
-          d.m_stubPtr   = &methodStub<T, TMethod>;
-          return d;
-        }
-        /** Creates a delegate given an object, and a plain function. */
-        static FunctionDelegate fromFunction(const void *obj,StubType func)
-        {
-          FunctionDelegate d;
-          d.m_objectPtr = obj;
-          d.m_stubPtr = func;
-          return d;
-        }
-
-        /** Invokes the function/method stored in the delegate */
-        TemplateVariant operator()(const std::vector<TemplateVariant> &args) const
-        {
-          return (*m_stubPtr)(m_objectPtr, args);
-        }
-
-      private:
-        const void* m_objectPtr;
-        StubType    m_stubPtr;
-
-        template <class T, TemplateVariant (T::*TMethod)(const std::vector<TemplateVariant> &) const>
-        static TemplateVariant methodStub(const void* objectPtr, const std::vector<TemplateVariant> &args)
-        {
-          T* p = (T*)(objectPtr);
-          return (p->*TMethod)(args);
-        }
+      None       = 0,
+      Bool       = 1,
+      Int        = 2,
+      String     = 3,
+      Struct     = 4,
+      List       = 5,
+      Function   = 6,
+      WeakStruct = 7
     };
 
     /** Constructs an invalid variant. */
     TemplateVariant() {}
 
     /** Constructs a new variant with a boolean value \a b. */
-    explicit TemplateVariant(bool b) { m_variant.set<bool>(b); }
+    explicit TemplateVariant(bool b) { m_variant = b; }
 
     /** Constructs a new variant with a integer value \a v. */
-    TemplateVariant(int v) { m_variant.set<int>(v); }
+    TemplateVariant(int v) { m_variant = v; }
+
+    /** Constructs a new variant with a integer value \a v. */
+    TemplateVariant(unsigned int v) { m_variant = static_cast<int>(v); }
+
+    /** Constructs a new variant with a integer value \a v.
+     *  We use SFINAE to avoid a compiler error in case size_t already matches the 'unsigned int' overload.
+     */
+    template<typename T,
+             typename std::enable_if<std::is_same<T,size_t>::value,T>::type* = nullptr
+            >
+    TemplateVariant(T v) { m_variant = static_cast<int>(v); }
 
     /** Constructs a new variant with a string value \a s. */
-    TemplateVariant(const char *s,bool raw=FALSE) : m_raw(raw) { m_variant.set<QCString>(s); }
+    TemplateVariant(const char *s,bool raw=FALSE) : m_raw(raw) { m_variant = QCString(s); }
 
     /** Constructs a new variant with a string value \a s. */
-    TemplateVariant(const QCString &s,bool raw=FALSE) : m_raw(raw) { m_variant.set<QCString>(s); }
+    TemplateVariant(const QCString &s,bool raw=FALSE) : m_raw(raw) { m_variant = s; }
 
     /** Constructs a new variant with a string value \a s. */
-    TemplateVariant(const std::string &s,bool raw=FALSE) : m_raw(raw) { m_variant.set<QCString>(s); }
+    TemplateVariant(const std::string &s,bool raw=FALSE) : m_raw(raw) { m_variant = QCString(s); }
 
     /** Constructs a new variant with a struct value \a s.
-     *  @note. The variant will hold a reference to the object.
+     *  @note. The variant will hold a counting reference to the object.
      */
-    TemplateVariant(TemplateStructIntf *s);
+    TemplateVariant(TemplateStructIntfPtr s) { m_variant = s; }
 
     /** Constructs a new variant with a list value \a l.
-     *  @note. The variant will hold a reference to the object.
+     *  @note. The variant will hold a counting reference to the object.
      */
-    TemplateVariant(TemplateListIntf *l);
+    TemplateVariant(TemplateListIntfPtr l) { m_variant = l; }
+
+    /** Constructs a new variant with a struct value \a s.
+     *  @note. The variant will hold a non-counting reference to the object.
+     */
+    TemplateVariant(TemplateStructIntfWeakPtr s) { m_variant = s; }
 
     /** Constructs a new variant which represents a method call
      *  @param[in] delegate FunctionDelegate object to invoke when
@@ -172,15 +175,15 @@ class TemplateVariant
      *  TemplateVariant::FunctionDelegate::fromFunction() to create
      *  FunctionDelegate objects.
      */
-    TemplateVariant(const FunctionDelegate &delegate) { m_variant.set<FunctionDelegate>(delegate); }
+    TemplateVariant(FunctionDelegate delegate) { m_variant = delegate; }
 
     /** Destroys the Variant object */
-    ~TemplateVariant();
+    ~TemplateVariant()  = default;
 
     /** Constructs a copy of the variant, \a v,
      *  passed as the argument to this constructor.
      */
-    TemplateVariant(const TemplateVariant &v);
+    TemplateVariant(const TemplateVariant &v) = default;
 
     /** Moves the contents of variant \a v into this variant.
      *  variant \a v will become invalid
@@ -188,7 +191,7 @@ class TemplateVariant
     TemplateVariant(TemplateVariant &&v);
 
     /** Assigns the value of the variant \a v to this variant. */
-    TemplateVariant &operator=(const TemplateVariant &v);
+    TemplateVariant &operator=(const TemplateVariant &v) = default;
 
     /** Move the value of the variant \a v into this variant.
      *  Variant \a v will become invalid */
@@ -197,33 +200,7 @@ class TemplateVariant
     /** Compares this QVariant with v and returns true if they are equal;
      *  otherwise returns false.
      */
-    bool operator==(TemplateVariant &other)
-    {
-      if (!m_variant.valid())
-      {
-        return FALSE;
-      }
-      if (isBool() && other.isBool())
-      {
-        return m_variant.get<bool>() == other.m_variant.get<bool>();
-      }
-      else if (isInt() && other.isInt())
-      {
-        return m_variant.get<int>() == other.m_variant.get<int>();
-      }
-      else if (isList() && other.isList())
-      {
-        return m_variant.get<TemplateListIntf*>() == other.m_variant.get<TemplateListIntf*>();
-      }
-      else if (isStruct() && other.isStruct())
-      {
-        return m_variant.get<TemplateStructIntf*>() == other.m_variant.get<TemplateStructIntf*>();
-      }
-      return toString()==other.toString();
-    }
-
-    QCString listToString() const;
-    QCString structToString() const;
+    bool operator==(TemplateVariant &other) const;
 
     /** Returns the variant as a string. */
     QCString toString() const;
@@ -235,51 +212,38 @@ class TemplateVariant
     int toInt() const;
 
     /** Returns TRUE if the variant holds a valid value, or FALSE otherwise */
-    constexpr bool isValid()    const { return m_variant.valid(); }
+    constexpr bool isValid()      const { return std::holds_alternative<std::monostate>(m_variant); }
     /** Returns TRUE if the variant holds a boolean value */
-    constexpr bool isBool()     const { return m_variant.is<bool>(); }
+    constexpr bool isBool()       const { return std::holds_alternative<bool>(m_variant); }
     /** Returns TRUE if the variant holds an integer value */
-    constexpr bool isInt()      const { return m_variant.is<int>(); }
+    constexpr bool isInt()        const { return std::holds_alternative<int>(m_variant); }
     /** Returns TRUE if the variant holds a string value */
-    constexpr bool isString()   const { return m_variant.is<QCString>(); }
+    constexpr bool isString()     const { return std::holds_alternative<QCString>(m_variant); }
     /** Returns TRUE if the variant holds a struct value */
-    constexpr bool isStruct()   const { return m_variant.is<TemplateStructIntf*>(); }
+    constexpr bool isStruct()     const { return std::holds_alternative<TemplateStructIntfPtr>(m_variant); }
     /** Returns TRUE if the variant holds a list value */
-    constexpr bool isList()     const { return m_variant.is<TemplateListIntf*>(); }
+    constexpr bool isList()       const { return std::holds_alternative<TemplateListIntfPtr>(m_variant); }
     /** Returns TRUE if the variant holds a function value */
-    constexpr bool isFunction() const { return m_variant.is<FunctionDelegate>(); }
+    constexpr bool isFunction()   const { return std::holds_alternative<FunctionDelegate>(m_variant); }
+    /** Returns TRUE if the variant holds a struct value */
+    constexpr bool isWeakStruct() const { return std::holds_alternative<TemplateStructIntfWeakPtr>(m_variant); }
 
     /** Returns the pointer to list referenced by this variant
      *  or 0 if this variant does not have list type.
      */
-    TemplateListIntf   *toList()
-    {
-      return isList() ? m_variant.get<TemplateListIntf*>() : nullptr;
-    }
-    const TemplateListIntf *toList() const
-    {
-      return isList() ? m_variant.get<TemplateListIntf*>() : nullptr;
-    }
+    TemplateListIntfPtr toList();
+    const TemplateListIntfPtr toList() const;
 
     /** Returns the pointer to struct referenced by this variant
      *  or 0 if this variant does not have struct type.
      */
-    TemplateStructIntf *toStruct()
-    {
-      return isStruct() ? m_variant.get<TemplateStructIntf*>() : nullptr;
-    }
-    const TemplateStructIntf *toStruct() const
-    {
-      return isStruct() ? m_variant.get<TemplateStructIntf*>() : nullptr;
-    }
+    TemplateStructIntfPtr toStruct();
+    const TemplateStructIntfPtr toStruct() const;
 
     /** Return the result of apply this function with \a args.
      *  Returns an empty string if the variant type is not a function.
      */
-    TemplateVariant call(const std::vector<TemplateVariant> &args)
-    {
-      return isFunction() ? m_variant.get<FunctionDelegate>()(args) : TemplateVariant();
-    }
+    TemplateVariant call(const std::vector<TemplateVariant> &args = std::vector<TemplateVariant>());
 
     /** Sets whether or not the value of the Variant should be
      *  escaped or written as-is (raw).
@@ -292,69 +256,21 @@ class TemplateVariant
      */
     constexpr bool raw() const { return m_raw; }
 
-    /** Symbolic names for the possible types that this variant can hold. */
-    enum class Type : size_t
-    {
-      None     = std::string::npos,
-      Bool     = 0,
-      Int      = 1,
-      String   = 2,
-      Struct   = 3,
-      List     = 4,
-      Function = 5
-    };
-
     /** Returns the type held by this variant */
     constexpr Type type() const { return static_cast<Type>(m_variant.index()); }
 
-    /** Return a string representation of the type of the value stored in the variant */
-    constexpr const char *typeAsString() const
-    {
-      switch (type())
-      {
-        case Type::None:       return "invalid";
-        case Type::Bool:       return "bool";
-        case Type::Int:        return "integer";
-        case Type::String:     return "string";
-        case Type::Struct:     return "struct";
-        case Type::List:       return "list";
-        case Type::Function:   return "function";
-      }
-      return "invalid";
-    }
+    /** Returns a string representation of this variant's type */
+    const char *typeAsString() const;
 
   private:
-    using VariantT = Variant<bool,                  // index==0: Type::Bool
-                             int,                   // index==1: Type::Int
-                             QCString,              // index==2: Type::String
-                             TemplateStructIntf*,   // index==3: Type::Struct
-                             TemplateListIntf*,     // index==4: Type::List
-                             FunctionDelegate       // index==5: Type::Function
-                            >;
+    QCString listToString() const;
+    QCString structToString() const;
+
     VariantT              m_variant;
     bool                  m_raw = false;
 };
 
-//------------------------------------------------------------------------
-
-template<class T> class TemplateAutoRef
-{
-  public:
-    TemplateAutoRef(T *obj) : m_obj(obj)
-    {
-      m_obj->addRef();
-    }
-   ~TemplateAutoRef()
-    {
-      m_obj->release();
-    }
-    T &operator*() const { return *m_obj; }
-    T *operator->() const { return m_obj; }
-    T *get() const { return m_obj; }
-
-  private:
-   T *m_obj;
-};
+using TemplateVariantList = std::vector<TemplateVariant>;
 
 //------------------------------------------------------------------------
 
@@ -369,7 +285,7 @@ class TemplateListIntf
     {
       public:
         /** Destructor for the iterator */
-        virtual ~ConstIterator() {}
+        virtual ~ConstIterator() = default;
         /** Moves iterator to the first element in the list */
         virtual void toFirst() = 0;
         /** Moves iterator to the last element in the list */
@@ -385,54 +301,46 @@ class TemplateListIntf
          */
         virtual bool current(TemplateVariant &v) const = 0;
     };
+    using ConstIteratorPtr = std::unique_ptr<ConstIterator>;
 
     /** Destroys the list */
-    virtual ~TemplateListIntf() {}
+    virtual ~TemplateListIntf() = default;
 
     /** Returns the number of elements in the list */
-    virtual uint count() const = 0;
+    virtual size_t count() const = 0;
 
     /** Returns the element at index position \a index. */
-    virtual TemplateVariant  at(uint index) const = 0;
+    virtual TemplateVariant  at(size_t index) const = 0;
 
     /** Creates a new iterator for this list.
      *  @note the user should call delete on the returned pointer.
      */
-    virtual TemplateListIntf::ConstIterator *createIterator() const = 0;
+    virtual TemplateListIntf::ConstIteratorPtr createIterator() const = 0;
 
-    /** Increase object's reference count */
-    virtual int addRef() = 0;
-
-    /** Decreases object's reference count, destroy object if 0 */
-    virtual int release() = 0;
 };
 
-/** @brief Default implementation of a context value of type list. */
-class TemplateList : public TemplateListIntf
+/** @brief Default implementation of a immutable context value of type list. */
+class TemplateImmutableList : public TemplateListIntf
 {
   public:
     // TemplateListIntf methods
-    virtual uint count() const;
-    virtual TemplateVariant at(uint index) const;
-    virtual TemplateListIntf::ConstIterator *createIterator() const;
-    virtual int addRef();
-    virtual int release();
+    virtual size_t count() const;
+    virtual TemplateVariant at(size_t index) const;
+    virtual TemplateListIntf::ConstIteratorPtr createIterator() const;
 
-    /** Creates an instance with ref count set to 0 */
-    static TemplateList *alloc();
+    /** Creates an instance and returns a shared pointer to it */
+    static TemplateListIntfPtr alloc(std::initializer_list<TemplateVariant> elements);
+    static TemplateListIntfPtr alloc(const std::vector<TemplateVariant> &elements);
 
-    /** Appends element \a v to the end of the list */
-    virtual void append(const TemplateVariant &v);
+    /** Creates a list */
+    TemplateImmutableList(std::initializer_list<TemplateVariant> elements);
+    TemplateImmutableList(const std::vector<TemplateVariant> &elements);
+    /** Destroys the list */
+    virtual ~TemplateImmutableList();
 
   private:
-    /** Creates a list */
-    TemplateList();
-    /** Destroys the list */
-   ~TemplateList();
-
-    friend class TemplateListConstIterator;
     class Private;
-    Private *p;
+    std::unique_ptr<Private> p;
 };
 
 //------------------------------------------------------------------------
@@ -442,7 +350,7 @@ class TemplateStructIntf
 {
   public:
     /** Destroys the struct */
-    virtual ~TemplateStructIntf() {}
+    virtual ~TemplateStructIntf() = default;
 
     /** Gets the value for a field name.
      *  @param[in] name The name of the field.
@@ -451,52 +359,46 @@ class TemplateStructIntf
 
     /** Return the list of fields. */
     virtual StringVector fields() const = 0;
-
-    /** Increase object's reference count */
-    virtual int addRef() = 0;
-
-    /** Decreases object's reference count, destroy object if 0 */
-    virtual int release() = 0;
 };
 
+//------------------------------------------------------------------------
 
-/** @brief Default implementation of a context value of type struct. */
-class TemplateStruct : public TemplateStructIntf
+/** @brief Default implementation of an immutable context value of type struct. */
+class TemplateImmutableStruct : public TemplateStructIntf
 {
   public:
     // TemplateStructIntf methods
     virtual TemplateVariant get(const QCString &name) const;
     virtual StringVector fields() const;
-    virtual int addRef();
-    virtual int release();
 
-    /** Creates an instance with ref count set to 0. */
-    static TemplateStruct *alloc();
+    using StructField = std::pair<const std::string,TemplateVariant>;
 
-    /** Sets the value the field of a struct
-     *  @param[in] name The name of the field.
-     *  @param[in] v The value to set.
+    /** Creates an instance and returns a shared pointer to it
+     *  @param fields the fields of the struct as key/value pairs.
      */
-    virtual void set(const QCString &name,const TemplateVariant &v);
+    static TemplateStructIntfPtr alloc(std::initializer_list<StructField> fields);
 
+    /** Creates a struct */
+    TemplateImmutableStruct(std::initializer_list<StructField> fields);
+    /** Destroys the struct */
+    virtual ~TemplateImmutableStruct();
 
   private:
-    /** Creates a struct */
-    TemplateStruct();
-    /** Destroys the struct */
-    virtual ~TemplateStruct();
 
     class Private;
-    Private *p;
+    std::unique_ptr<Private> p;
 };
 
+
 //------------------------------------------------------------------------
 
 /** @brief Interface used to escape characters in a string */
 class TemplateEscapeIntf
 {
   public:
-    virtual ~TemplateEscapeIntf() {}
+    virtual ~TemplateEscapeIntf() = default;
+    /** Create a copy of the escape filter */
+    virtual std::unique_ptr<TemplateEscapeIntf> clone() = 0;
     /** Returns the \a input after escaping certain characters */
     virtual QCString escape(const QCString &input) = 0;
     /** Setting tabbing mode on or off (for LaTeX) */
@@ -509,7 +411,9 @@ class TemplateEscapeIntf
 class TemplateSpacelessIntf
 {
   public:
-    virtual ~TemplateSpacelessIntf() {}
+    virtual ~TemplateSpacelessIntf() = default;
+    /** Create a copy of the spaceless filter */
+    virtual std::unique_ptr<TemplateSpacelessIntf> clone() = 0;
     /** Returns the \a input after removing redundant whitespace */
     virtual QCString remove(const QCString &input) = 0;
     /** Reset filter state */
@@ -530,7 +434,7 @@ class TemplateSpacelessIntf
 class TemplateContext
 {
   public:
-    virtual ~TemplateContext() {}
+    virtual ~TemplateContext() = default;
 
     /** Push a new scope on the stack. */
     virtual void push() = 0;
@@ -567,12 +471,12 @@ class TemplateContext
     /** Sets the interface that will be used for escaping the result
      *  of variable expansion before writing it to the output.
      */
-    virtual void setEscapeIntf(const QCString &extension, TemplateEscapeIntf *intf) = 0;
+    virtual void setEscapeIntf(const QCString &extension, std::unique_ptr<TemplateEscapeIntf> intf) = 0;
 
     /** Sets the interface that will be used inside a spaceless block
      *  to remove any redundant whitespace.
      */
-    virtual void setSpacelessIntf(TemplateSpacelessIntf *intf) = 0;
+    virtual void setSpacelessIntf(std::unique_ptr<TemplateSpacelessIntf> intf) = 0;
 };
 
 //------------------------------------------------------------------------
@@ -584,7 +488,7 @@ class Template
 {
   public:
     /** Destructor */
-    virtual ~Template() {}
+    virtual ~Template() = default;
 
     /** Renders a template instance to a stream.
      *  @param[in] ts The text stream to write the results to.
@@ -609,12 +513,7 @@ class TemplateEngine
     /** Creates a new context that can be using to render a template.
      *  @see Template::render()
      */
-    TemplateContext *createContext() const;
-
-    /** Destroys a context created via createContext().
-     *  @param[in] ctx The context.
-     */
-    void destroyContext(TemplateContext *ctx);
+    std::unique_ptr<TemplateContext> createContext() const;
 
     /** Creates a new template whose contents are in a file.
      *  @param[in] fileName The name of the file containing the template data
@@ -650,7 +549,7 @@ class TemplateEngine
     QCString outputExtension() const;
 
     class Private;
-    Private *p;
+    std::unique_ptr<Private> p;
 };
 
 /** @} */
