@@ -36,12 +36,12 @@
 // files frees the maintainer from thinking about whether the
 // first, the second, or both files should be included or not, and
 // why.  This holds namely for localized translators because their
-// base class is changed occasionaly to adapter classes when the
+// base class is changed occasionally to adapter classes when the
 // Translator class changes the interface, or back to the
 // Translator class (by the local maintainer) when the localized
 // translator is made up-to-date again.
 
-class TranslatorChinesetraditional : public Translator
+class TranslatorChinesetraditional : public TranslatorAdapter_1_8_15
 {
   public:
 
@@ -71,9 +71,29 @@ class TranslatorChinesetraditional : public Translator
      * the empty string is returned in this implementation.
      */
     virtual QCString latexLanguageSupportCommand()
+    {
+      return "\\usepackage{CJKutf8}\n";
+    }
+    virtual QCString latexFontenc()
     {
       return "";
     }
+    virtual QCString latexDocumentPre()
+    {
+      return "\\begin{CJK}{UTF8}{min}\n";
+    }
+    virtual QCString latexDocumentPost()
+    {
+      return "\\end{CJK}\n";
+    }
+    virtual QCString trISOLang()
+    {
+      return "zh-Hant";
+    }
+    virtual QCString getLanguageString()
+    {
+      return "0x404 Chinese (Taiwan)";
+    }
 
     // --- Language translation methods -------------------
 
@@ -104,7 +124,7 @@ class TranslatorChinesetraditional : public Translator
     /*! header that is put before the list of member attributes. */
     virtual QCString trMemberDataDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "欄位說明文件";
       }
@@ -137,9 +157,9 @@ class TranslatorChinesetraditional : public Translator
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
-    virtual QCString trGeneratedAutomatically(const char *s)
+    virtual QCString trGeneratedAutomatically(const QCString &s)
     { QCString result="本文件由Doxygen";
-      if (s) result+=(QCString)" 自 "+s;
+      if (!s.isEmpty()) result+=" 自 "+s;
       result+=" 的原始碼中自動產生.";
       return result;
     }
@@ -171,7 +191,7 @@ class TranslatorChinesetraditional : public Translator
     /*! This is put above each page as a link to the list of annotated classes */
     virtual QCString trCompoundList()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "資料結構";
       }
@@ -188,7 +208,7 @@ class TranslatorChinesetraditional : public Translator
     /*! This is put above each page as a link to all members of compounds. */
     virtual QCString trCompoundMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "資料欄位";
       }
@@ -201,7 +221,7 @@ class TranslatorChinesetraditional : public Translator
     /*! This is put above each page as a link to all members of files. */
     virtual QCString trFileMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "全域資料";
       }
@@ -241,7 +261,7 @@ class TranslatorChinesetraditional : public Translator
     virtual QCString trCompoundListDescription()
     {
 
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "這是附帶簡略說明的資料結構:";
       }
@@ -260,7 +280,7 @@ class TranslatorChinesetraditional : public Translator
       {
         result+="文件化過";
       }
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="結構及聯合型態欄位";
       }
@@ -271,7 +291,7 @@ class TranslatorChinesetraditional : public Translator
       result+=", 並且帶有連結至";
       if (!extractAll)
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="每個欄位的結構/聯合型態說明文件:";
         }
@@ -282,7 +302,7 @@ class TranslatorChinesetraditional : public Translator
       }
       else
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="這些結構/聯合型態所屬:";
         }
@@ -300,7 +320,7 @@ class TranslatorChinesetraditional : public Translator
       QCString result="這是全部";
       if (!extractAll) result+="文件化的";
 
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="函式，變數，定義，列舉，及型態定義";
       }
@@ -352,7 +372,7 @@ class TranslatorChinesetraditional : public Translator
      */
     virtual QCString trCompoundIndex()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "資料結構索引";
       }
@@ -379,10 +399,14 @@ class TranslatorChinesetraditional : public Translator
      */
     virtual QCString trClassDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "資料結構說明文件";
       }
+      else if (Config_getBool(OPTIMIZE_OUTPUT_VHDL))
+      {
+          return trDesignUnitDocumentation();
+      }
       else
       {
         return "類別說明文件";
@@ -401,12 +425,6 @@ class TranslatorChinesetraditional : public Translator
     virtual QCString trExampleDocumentation()
     { return "範例說明文件"; }
 
-    /*! This is used in LaTeX as the title of the chapter containing
-     *  the documentation of all related pages.
-     */
-    virtual QCString trPageDocumentation()
-    { return "頁面說明文件"; }
-
     /*! This is used in LaTeX as the title of the document */
     virtual QCString trReferenceManual()
     { return "參考手冊"; }
@@ -488,7 +506,7 @@ class TranslatorChinesetraditional : public Translator
      */
     virtual QCString trCompounds()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "資料結構";
       }
@@ -501,24 +519,20 @@ class TranslatorChinesetraditional : public Translator
     /*! This is used in the standard footer of each page and indicates when
      *  the page was generated
      */
-    virtual QCString trGeneratedAt(const char *date,const char *projName)
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
-      QCString result=(QCString)"產生日期:"+date;
-      if (projName) result+=(QCString)", 專案:"+projName;
-      result+=(QCString)", 產生器:";
+      QCString result="產生日期:"+date;
+      if (!projName.isEmpty()) result+=", 專案:"+projName;
+      result+=", 產生器:";
       return result;
     }
 
     /*! this text is put before a class diagram */
-    virtual QCString trClassDiagram(const char *clName)
+    virtual QCString trClassDiagram(const QCString &clName)
     {
-      return (QCString)"類別"+clName+"的繼承圖:";
+      return "類別"+clName+"的繼承圖:";
     }
 
-    /*! this text is generated when the \\internal command is used. */
-    virtual QCString trForInternalUseOnly()
-    { return "僅供內部使用."; }
-
     /*! this text is generated when the \\warning command is used. */
     virtual QCString trWarning()
     { return "警告"; }
@@ -589,11 +603,11 @@ class TranslatorChinesetraditional : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! used as the title of the HTML page of a class/struct/union */
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
-      QCString result=(QCString)clName+" ";
+      QCString result=clName+" ";
       switch(compType)
       {
         case ClassDef::Class:      result+=" 類別"; break;
@@ -611,7 +625,7 @@ class TranslatorChinesetraditional : public Translator
     }
 
     /*! used as the title of the HTML page of a file */
-    virtual QCString trFileReference(const char *fileName)
+    virtual QCString trFileReference(const QCString &fileName)
     {
       QCString result=fileName;
       result+=" 檔案參考文件";
@@ -619,7 +633,7 @@ class TranslatorChinesetraditional : public Translator
     }
 
     /*! used as the title of the HTML page of a namespace */
-    virtual QCString trNamespaceReference(const char *namespaceName)
+    virtual QCString trNamespaceReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+=" 命名空間(Namespace)參考文件";
@@ -749,11 +763,10 @@ class TranslatorChinesetraditional : public Translator
     /*! This is put at the bottom of a class documentation page and is
      *  followed by a list of files that were used to generate the page.
      */
-    virtual QCString trGeneratedFromFiles(ClassDef::CompoundType compType,
-        bool single)
+    virtual QCString trGeneratedFromFiles(ClassDef::CompoundType compType,bool)
     { // here s is one of " Class", " Struct" or " Union"
       // single is true implies a single file
-      QCString result=(QCString)"此";
+      QCString result="此";
       switch(compType)
       {
         case ClassDef::Class:      result+="類別(class)"; break;
@@ -766,7 +779,7 @@ class TranslatorChinesetraditional : public Translator
         default: break;
       }
       result+=" 文件是由下列檔案中產生";
-      if (single) result+=":"; else result+=":";
+      result+=":";
       return result;
     }
 
@@ -816,14 +829,14 @@ class TranslatorChinesetraditional : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    virtual QCString trCollaborationDiagram(const char *clName)
+    virtual QCString trCollaborationDiagram(const QCString &clName)
     {
-      return (QCString)""+clName+"的合作圖:";
+      return ""+clName+"的合作圖:";
     }
     /*! this text is put before an include dependency graph */
-    virtual QCString trInclDepGraph(const char *fName)
+    virtual QCString trInclDepGraph(const QCString &fName)
     {
-      return (QCString)""+fName+"的包含相依圖:";
+      return ""+fName+"的包含相依圖:";
     }
     /*! header that is put before the list of constructor/destructors. */
     virtual QCString trConstructorDocumentation()
@@ -896,7 +909,7 @@ class TranslatorChinesetraditional : public Translator
     }
     virtual QCString trPublicAttribs()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "資料欄位";
       }
@@ -1092,7 +1105,7 @@ class TranslatorChinesetraditional : public Translator
     /*! Used for Java classes in the summary section of Java packages */
     virtual QCString trClasses()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "資料結構";
       }
@@ -1102,14 +1115,9 @@ class TranslatorChinesetraditional : public Translator
       }
     }
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
-      return (QCString)"Package "+name;
-    }
-    /*! Title of the package index page */
-    virtual QCString trPackageList()
-    {
-      return "Package列表";
+      return "Package "+name;
     }
     /*! The description of the package index page */
     virtual QCString trPackageListDescription()
@@ -1351,14 +1359,18 @@ class TranslatorChinesetraditional : public Translator
     /*! Used as a heading for a list of Java class functions with package
      * scope.
      */
-    virtual QCString trPackageMembers()
+    virtual QCString trPackageFunctions()
     {
       return "Package 函數列表";
     }
+    virtual QCString trPackageMembers()
+    {
+      return "Package 成員列表";
+    }
     /*! Used as a heading for a list of static Java class functions with
      *  package scope.
      */
-    virtual QCString trStaticPackageMembers()
+    virtual QCString trStaticPackageFunctions()
     {
       return "靜態 Package 函數列表";
     }
@@ -1470,17 +1482,10 @@ class TranslatorChinesetraditional : public Translator
     virtual QCString trDirectories()
     { return "目錄"; }
 
-    /*! This returns a sentences that introduces the directory hierarchy.
-     *  and the fact that it is sorted alphabetically per level
-     */
-    virtual QCString trDirDescription()
-    { return "這個目錄階層經過簡略的字母排序: ";
-    }
-
     /*! This returns the title of a directory page. The name of the
      *  directory is passed via \a dirName.
      */
-    virtual QCString trDirReference(const char *dirName)
+    virtual QCString trDirReference(const QCString &dirName)
     { QCString result=dirName; result+=" 目錄參考文件"; return result; }
 
     /*! This returns the word directory with or without starting capital
@@ -1599,11 +1604,11 @@ class TranslatorChinesetraditional : public Translator
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
-      QCString result=(QCString)clName;
+      QCString result=clName;
       switch(compType)
       {
         case ClassDef::Class:      result+="模組"; break;
@@ -1620,7 +1625,7 @@ class TranslatorChinesetraditional : public Translator
       return result;
     }
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+="模組參考文件";
@@ -1670,7 +1675,7 @@ class TranslatorChinesetraditional : public Translator
         bool /* single */)
     { // here s is one of " Module", " Struct" or " Union"
       // single is true implies a single file
-      QCString result=(QCString)"這個";
+      QCString result="這個";
       switch(compType)
       {
         case ClassDef::Class:      result+="模組"; break;
@@ -1715,7 +1720,7 @@ class TranslatorChinesetraditional : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! directory relation for \a name */
-    virtual QCString trDirRelation(const char *name)
+    virtual QCString trDirRelation(const QCString &name)
     {
       return QCString(name)+" 關連";
     }
@@ -1752,18 +1757,18 @@ class TranslatorChinesetraditional : public Translator
      *  table is shown. The heading for the first column mentions the
      *  source file that has a relation to another file.
      */
-    virtual QCString trFileIn(const char *name)
+    virtual QCString trFileIn(const QCString &name)
     {
-      return (QCString)"檔案在"+name;
+      return "檔案在"+name;
     }
 
     /*! when clicking a directory dependency label, a page with a
      *  table is shown. The heading for the second column mentions the
      *  destination file that is included.
      */
-    virtual QCString trIncludesFileIn(const char *name)
+    virtual QCString trIncludesFileIn(const QCString &name)
     {
-      return (QCString)"含入檔案在"+name;
+      return "含入檔案在"+name;
     }
 
     /** Compiles a date string.
@@ -1806,7 +1811,7 @@ class TranslatorChinesetraditional : public Translator
     { return "版權聲明"; }
 
     /*! Header for the graph showing the directory dependencies */
-    virtual QCString trDirDepGraph(const char *name)
+    virtual QCString trDirDepGraph(const QCString &name)
     { return QCString(name)+"的目錄關連圖"+":"; }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1834,11 +1839,11 @@ class TranslatorChinesetraditional : public Translator
     }
 
     /*! Header of a Java enum page (Java enums are represented as classes). */
-    virtual QCString trEnumReference(const char *name)
+    virtual QCString trEnumReference(const QCString &name)
     { return QCString(name)+" 列舉型態參考"; }
 
     /*! Used for a section containing inherited members */
-    virtual QCString trInheritedFrom(const char *members,const char *what)
+    virtual QCString trInheritedFrom(const QCString &members,const QCString &what)
     { return QCString(members)+" 繼承自 "+what; }
 
     /*! Header of the sections with inherited members specific for the
@@ -1867,7 +1872,7 @@ class TranslatorChinesetraditional : public Translator
      */
     virtual QCString trProvidedByCategory()
     {
-      return "由 @1 分類所提供.";
+      return "由 @0 分類所提供.";
     }
 
     /*! Used in a method of an Objective-C category that extends a class.
@@ -1876,7 +1881,7 @@ class TranslatorChinesetraditional : public Translator
      */
     virtual QCString trExtendsClass()
     {
-      return "延伸 @1 類別 .";
+      return "延伸 @0 類別 .";
     }
 
     /*! Used as the header of a list of class methods in Objective-C.
@@ -1902,14 +1907,6 @@ class TranslatorChinesetraditional : public Translator
       return "方法文件";
     }
 
-    /*! Used as the title of the design overview picture created for the
-     *  VHDL output.
-     */
-    virtual QCString trDesignOverview()
-    {
-      return "設計概述";
-    }
-
 //////////////////////////////////////////////////////////////////////////
 // new since 1.8.4
 //////////////////////////////////////////////////////////////////////////
@@ -1927,47 +1924,45 @@ class TranslatorChinesetraditional : public Translator
     { return "常數群組"; }
 
     /** UNO IDL constant groups */
-    virtual QCString trConstantGroupReference(const char *namespaceName)
+    virtual QCString trConstantGroupReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+="常數群組參考";
       return result;
     }
     /** UNO IDL service page title */
-    virtual QCString trServiceReference(const char *sName)
+    virtual QCString trServiceReference(const QCString &sName)
     {
-      QCString result=(QCString)sName;
+      QCString result=sName;
       result+="服務參考";
       return result;
     }
     /** UNO IDL singleton page title */
-    virtual QCString trSingletonReference(const char *sName)
+    virtual QCString trSingletonReference(const QCString &sName)
     {
-      QCString result=(QCString)sName;
+      QCString result=sName;
       result+="Singleton參考";
       return result;
     }
     /** UNO IDL service page */
-    virtual QCString trServiceGeneratedFromFiles(bool single)
+    virtual QCString trServiceGeneratedFromFiles(bool)
     {
       // single is true implies a single file
-      QCString result=(QCString)"本服務的文件由以下的檔案"
+      QCString result="本服務的文件由以下的檔案"
                                 "所產生";
-      if (single) result+=":"; else result+=":";
+      result+=":";
       return result;
     }
     /** UNO IDL singleton page */
-    virtual QCString trSingletonGeneratedFromFiles(bool single)
+    virtual QCString trSingletonGeneratedFromFiles(bool)
     {
       // single is true implies a single file
-      QCString result=(QCString)"本singleton的文件由下面的檔案"
+      QCString result="本singleton的文件由下面的檔案"
                                 "所產生";
-      if (single) result+=":"; else result+=":";
+      result+=":";
       return result;
     }
 
-//////////////////////////////////////////////////////////////////////////
-
 };
 
 #endif
